"""datetimeutil converts between different datetime related objects.

In this module, make sure we don't use the tzinfo
parameter in the constructor datetime.datetime. Instead,
always gets a timezone from pytz, then do localize (and
normalize after that).
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import re
import time

from dateutil import parser as dateutilparser
import pytz
import six
import tzlocal

ISO_FORMAT= "%Y-%m-%dT%H:%M:%S%z"
ISO_FORMAT_WITH_TZ_NAME = "%Y-%m-%dT%H:%M:%S%z%Z"

LOCAL_TIMEZONE = tzlocal.get_localzone()

# Mon Jan 2 15:04:05 -0700 MST 2006
_mst_timezone = pytz.timezone("US/Mountain")
GO_REFERENCE_TIME = _mst_timezone.normalize(
    _mst_timezone.localize(
        datetime.datetime(
            2006,
            1,
            2,
            15,
            4,
            5,
        )
    )
)

TIMESTAMP_REGEX = re.compile(r"((?<=\D)|^)(\d{10,19})((?=\D)|$)")

_timestamp_units = ["s", "ms", "us", "ns"]


def real_localize(dt, tz):
    """Attach timezone info to a datetime object.

    This is basically tz.normalize(tz.localize(dt))

    Args:
        dt: A datetime.
        tz: A pytz timezone.

    Returns:
        A datetime object which is a clone of the given dt, with the given
        timezone info attached.
    """
    return tz.normalize(tz.localize(dt))


def convert_timezone(dt, tz=LOCAL_TIMEZONE):
    """Converts a datetime object to a specific timezone.

    By default, the datetime object is converted into local timezone."""
    return tz.normalize(dt.astimezone(tz))


def datetime_to_timestamp(dt):
    """Converts datetime to a UNIX timestamp.

    Args:
        dt: A datetime. The datetime to be converted.

    Returns:
        An integer that represents the converted UNIX timestamp in seconds.
    """
    # time.mktime can only take local time, so need to convert
    # dt to local timezone first
    dt = convert_timezone(dt, tz=LOCAL_TIMEZONE)
    return time.mktime(dt.timetuple())


def timestamp_to_local_datetime(timestamp):
    """Converts timestamp to a datetime with local timezone info.

    Args:
        timestamp: An int. The UNIX timestamp in seconds to be converted.

    Returns:
        The converted datetime object with local timezone info.
    """
    dt = datetime.datetime.fromtimestamp(timestamp, LOCAL_TIMEZONE)
    return dt


def number_to_local_datetime(number):
    """Guesses the units of a timestamp.

    Args:
        number: An int. The number that is assumed to be a UNIX timestamp
            without units specified.

    Returns:
        A tuple of (datetime, str). The first element is the datetime object
        for the timestamp with local one info. The second element is the guessed
        unit, which can be s, ms, us, or ns. Raise ValueError if a valid
        conversion cannot be done.
    """
    i = 0
    n = number
    while n > 10 ** 10:
        n /= 1000
        i += 1
        if i >= len(_timestamp_units):
            raise ValueError("number %s is too big" % number)

    dt = datetime.datetime.fromtimestamp(n, LOCAL_TIMEZONE)
    return dt, _timestamp_units[i]


# A tuple of 2-tuple's. The first element of a 2-tuple is the format to match
# against a string to extract the datetime, the element is the conversion
# method that probably converts the datetime object to local time zone.
_COMMON_DATETIME_FORMATS_AND_LAMBDA = (
    # %z only works in Python 3.2 or later
    # (ISO_FORMAT, lambda dt: dt.astimezone(LOCAL_TIMEZONE)),
    # %z only works in Python 3.2 or later
    # (ISO_FORMAT_WITH_TZ_NAME, lambda dt: dt.astimezone(LOCAL_TIMEZONE)),
    (
        "%Y-%m-%dT%H:%M:%S.%fZ",
        lambda dt: dt.replace(tzinfo=pytz.utc).astimezone(LOCAL_TIMEZONE),
    ),
)

def guess_local_datetime(s):
    """Guesses the datetime for a given string.

    Args:
        s: A string. A date time with unspecified format.

    Returns:
        A datetime object for the given string in local time zone.

    Raises:
        ValueError if the string cannot be parsed as a datetime object.
    """
    for dt_format, func in _COMMON_DATETIME_FORMATS_AND_LAMBDA:
        try:
            func(datetime.datetime.strptime(s, dt_format))
        except ValueError:
            continue

    dt = dateutilparser.parse(s)
    if dt.tzinfo:
        return dt.astimezone(LOCAL_TIMEZONE)
    else:
        return LOCAL_TIMEZONE.localize(dt)


def strftime(format, mixed, localize=False):
    """Formats a time to a string.

    Just like time.strftime
    (https://docs.python.org/2/library/time.html#time.strftime) but this function
    can take different types of "time" object. It can take a timestamp in
    seconds, a datetime object, or a struct_time
    (https://docs.python.org/2/library/time.html#time.struct_time).

    If mixed is a datetime with a timezone, the result of strftime
    """
    dt = None
    if type(mixed) in six.integer_types or type(mixed) == float:
        dt = timestamp_to_local_datetime(mixed)
    elif type(mixed) == datetime.datetime:
        dt = mixed
    elif type(mixed) == time.struct_time:
        dt = timestamp_to_local_datetime(time.mktime(mixed))

    return dt.strftime(format)



#!/usr/bin/env python
"""Prints the top files in terms of sizes.

Prints the top files in terms of sizes under a directory or its subdirectories
in terms of the size


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from heapq import heappush, heappop, heapreplace
import os
import sys


class TopSizeFilesFinder(object):
    def __init__(self, num):
        """
        num - Top num files in terms of the size
        """
        self.num = num
        self.heap_size = num + 1

    def run(self, dirpath):
        heap = []

        join = os.path.join
        getsize = os.path.getsize

        for root, dirs, files in os.walk(dirpath):
            for f in files:
                filepath = join(root, f)
                size = getsize(filepath)
                self._add_to_heap(heap, filepath, size)

        # take out the items in heap which will not be included in the top
        # files
        while len(heap) > self.num:
            heappop(heap)

        # reverse the heap with a stack since it's a min heap
        stack = []
        while heap:
            stack.append(heappop(heap))

        num_size_digits = 0
        if stack:
            num_size_digits = len(str(stack[-1][0]))
        while stack:
            size, filepath = stack.pop()
            print("{0:{1}d}\t{2}".format(size, num_size_digits, filepath))

    def _add_to_heap(self, heap, filepath, size):
        if len(heap) < self.heap_size:
            heappush(heap, (size, filepath))
        else:
            heapreplace(heap, (size, filepath))


def parse_args(cmd_args):
    description = """
        Print the top files under a directory or its subdirectories in terms
        of the size
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "path"
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = TopSizeFilesFinder(10)
    app.run(args.path)


if __name__ == '__main__':
    main()



def min_edit_distance_dp(s1, s2):
    if len(s1) < len(s2):
        # longer string at the left
        s1, s2 = s2, s1

    dists = [0] * (1 + len(s2))
    dists2 = [0] * (1 + len(s2))

    for i in range(len(s1) + 1):
        for j in range(len(s2) + 1):
            if i == 0:
                dists[j] = j
            elif j == 0:
                dists[j] = i
            else:
                if s1[i - 1] == s2[j - 1]:
                    dists[j] = dists2[j - 1]
                else:
                    dists[j] = 1 + min([
                        dists2[j],
                        dists[j - 1],
                        dists2[j - 1],
                    ])
        dists, dists2 = dists2, dists

    return dists2[-1]



#!/usr/bin/env python
"""A script to scrape Amazon product reviews from the web page.


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import random
import sys
import time
import urllib
import urlparse

import requests
from lxml import html

from sorno import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()


class App(object):
    HEADERS = {
        'user-agent': 'Mozilla/5.0 ;Windows NT 6.1; WOW64; Trident/7.0; rv:11.0; like Gecko',
    }

    def __init__(self, url, stop_at=-1):
        self.url = url
        self.stop_at = stop_at

    def run(self):
        _log.info("Given url: %s", self.url)
        url, cur_page_num = self.get_main_url_and_page_number(self.url)
        _log.info("Main url: %s", url)

        if self.stop_at > 0 and self.stop_at < cur_page_num:
            _log.info("Not fetching page number %s", cur_page_num)
            return

        _log.info("Fetch page %d", cur_page_num)
        initial_page_tree = self.get_tree_from_url(url)

        prev_page_items = None
        cur_page_items = self.get_items_from_page_tree(initial_page_tree)
        all_items = list(cur_page_items)

        while prev_page_items != cur_page_items:
            # sleep a little bit to avoid being characterized as a bot
            time.sleep(random.uniform(0.5, 2))

            prev_page_items = cur_page_items
            cur_page_num += 1
            if self.stop_at > 0 and self.stop_at < cur_page_num:
                _log.info("Not fetching page number %s", cur_page_num)
                break

            new_url = url + "?pageNumber=" + str(cur_page_num)

            _log.info("Fetch page %s", cur_page_num)
            cur_page_items = self.get_items_from_page_tree(
                self.get_tree_from_url(new_url)
            )
            _log.debug("%s items fetched", len(cur_page_items))

            all_items.extend(cur_page_items)

        for item in all_items:
            review_content = item['review']
            print(review_content.encode('utf8'))
            print("-" * 70)

    def get_main_url_and_page_number(self, url):
        """Get the product reviews page url and the starting page number.

        Args:
            url (str): The url for the product either the product page or the
                product reviews page.

        Returns:
            Returns the product reviews page url and the starting page number.
            This method will try to get the product reviews page, without all
            the junk queries.
        """
        url = self._ensure_product_reviews_url(url)

        n = 1
        parsed = urlparse.urlparse(url)

        query = parsed.query
        query_list = urlparse.parse_qsl(query)
        # capture the pageNumber in query, and leave other as is
        for k, v in query_list:
            if k == "pageNumber":
                n = int(v)

        modified_parsed = urlparse.ParseResult(
            scheme=parsed.scheme,
            netloc=parsed.netloc,
            path=parsed.path,
            params=parsed.params,
            # not setting the query since it's really not needed"
            query=None,
            fragment=parsed.fragment,
        )
        return modified_parsed.geturl(), n

    def _ensure_product_reviews_url(self, url):
        """Ensures the url is a product reviews page.

        The url maybe referring to the product page not the reviews, so we
        need to get the product reviews page if necessary.
        """
        if "/product-reviews/" in url:
            return url
        else:
            _log.info(
                "It's not a \"All customer reviews\" url, will try to fetch" +
                    " the correct one"
            )
            tree = self.get_tree_from_url(url)
            reviews_anchors = tree.xpath(
                "//*[@id='revF']/div/a[contains(@href, '/product-reviews/')]"
            )
            product_reviews_url = None
            for anchor in reviews_anchors:
                _log.info(
                    "Potential product reviews url: %s",
                    anchor.attrib['href'],
                )
                if not product_reviews_url:
                    product_reviews_url = anchor.attrib['href']

            url_query = urlparse.urlparse(product_reviews_url).query
            if not url_query:
                product_reviews_url += "?pageNumber=1"

            return product_reviews_url

    def get_tree_from_url(self, url):
        _log.debug("Fetch from url [%s]", url)
        website_text = requests.get(url, headers=self.HEADERS).text
        return html.fromstring(website_text)

    def get_items_from_page_tree(self, tree):
        """Get all review items for the page

        Args:
            tree: The page's dom tree.

        Returns:
            A list of items. Each item is a dictionary with the following
            fields (each key is a str):
                review (str): the review content
        """
        review_elements = self.get_reviews_from_node(tree)

        reviews = []
        for review_element in review_elements:
            review = {}
            review['review'] = self.get_text_from_element(review_element)
            reviews.append(review)
        return reviews

    def get_reviews_from_node(self, node):
        reviews = node.xpath("//span[@class='a-size-base review-text']")
        return reviews

    def get_text_from_element(self, node):
        """
        Return a plain text representation of an html node.
        """
        text_segments = []
        self._collect_text_from_element(node, text_segments)
        return "".join(text_segments)

    def _collect_text_from_element(self, node, text_segments):
        """
        Collect text from node and all its children recursively and put into
        text_segments as a list of strings.
        """
        if node.tag.lower() == "br":
            text_segments.append(os.linesep)

        if node.text:
            text_segments.append(node.text)

        for child in node:
            self._collect_text_from_element(child, text_segments)

        if node.tail:
            text_segments.append(node.tail)


def parse_args(cmd_args):
    description = """
A script to scrape Amazon product reviews
    """
    parser = argparse.ArgumentParser(
        description=description,
        # formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--stop-at",
        type=int,
        default=-1,
        help="Stop fetching more reviews at this page",
    )
    parser.add_argument(
        "url",
        help="The url that point to all reviews of an Amazon product. You"
            + " probably want to single-quote the url when running this"
            + " script in the command line because the url probably contains"
            + " shell characters. An example of a url is:"
            + " http://www.amazon.com/Ito-En-Beverage-Unsweetened-Bottles/product-reviews/B0017T2MWW/ref=cm_cr_dp_see_all_summary?ie=UTF8&showViewpoints=1&sortBy=byRankDescending",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger("PLAIN")

    app = App(args.url, stop_at=args.stop_at)
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_protobuf_to_dict.py converts text format of protobufs to python dict.

The script launches ipython for you to play with the parsed python dict.


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
from pprint import pprint
import re
import sys

from sorno import debuggingutil
from sorno import sornomisc


class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        with open(self.args.file) as f:
            content = f.read().strip()

        # combine all the fields into one protobuf that has the field "res" and
        # the field's value is the given protobufs read from the file
        obj_str = "res {\n%s\n}" % content

        # make any protobuf field key which has protobuf value to be a dict key
        # fld {...} => "fld": {...}
        obj_str = re.sub(r'(\S+) \{', r'"\1": {', obj_str)

        # make any non-protobuf field to be a dict key
        # fld: ... => "fld": ...
        obj_str = re.sub(r'([^\s"]+): ', r'"\1": ', obj_str)

        obj_str, s = protobuf_str_to_nested_list_str(obj_str, True)

        print("Your object before python eval:")
        print(obj_str)

        header = "Your object is in the variable: obj"
        header += "\nThe following functions are available: "
        functions = ("pprint",)
        header += ",".join(sorted(functions))

        protobuf = eval(obj_str)

        print("Your object before converting lists to python dict")
        pprint(protobuf)
        obj = protobuf_nested_ls_to_obj(protobuf)

        obj = obj.res

        debuggingutil.ipython_here(header=header)

        return 0


def protobuf_nested_ls_to_obj(mixed):
    if type(mixed) == list:
        if (type(mixed[0]) == list):
            return [protobuf_nested_ls_to_obj(protobuf) for protobuf in mixed]
        else:
            return sornomisc.Bunch(
                    **{
                        mixed[0]: process_protobuf_fields(mixed[1])
                    }
            )
    else:
        return mixed


def process_protobuf_fields(fields):
    d = {}
    for k, v in fields:
        if type(v) == list:
            processed = process_protobuf_fields(v)
        else:
            processed = v

        if k in d:
            if type(d[k]) == list:
                d[k].append(processed)
            else:
                d[k] = [d[k], processed]
        else:
            d[k] = processed

    return sornomisc.Bunch(**d)


def protobuf_str_to_nested_list_str(s, inside_protobuf):
    new_str = ""
    w = ""
    in_quote = False
    while s:
        c = s[0]
        if c == ":":
            if in_quote:
                w += c
            elif s.startswith(": {"):
                # we have a key in "w", and the value is a protobuf
                val, s = protobuf_str_to_nested_list_str(s[3:], True)
                new_str += ",[%s, [%s]]" % (w, val)
                w = ""
                continue
            else:
                # we have a key in "w", and the value is not protobuf
                val, s = protobuf_str_to_nested_list_str(s[2:], False)
                new_str += ",[%s, %s]" % (w, val)
                w = ""
                continue
        elif c == "}":
            if in_quote:
                w += c
            else:
                if w:
                    # we are at the last value of the last field of a protobuf
                    return w, s[1:]
                else:
                    # we are done with this protobuf
                    return new_str.lstrip(','), s[1:]
        elif c == '"':
            in_quote = not in_quote
            w += c
        elif c == ' ' or c == '\n':
            if in_quote:
                w += c
            elif w:
                if not inside_protobuf:
                    return w, s[1:]
        else:
            w += c

        s = s[1:]

    if w:
        new_str += w
    return new_str.lstrip(','), s


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "file"
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



"""dictutil provides additional classes or functions for python dicts
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from collections import defaultdict

class DefaultDictWithEnhancedFactory(defaultdict):
    """A dict with a default factory that takes the missing key"""
    def __missing__(self, key):
        return self.default_factory(key)



#!/usr/bin/python
"""Scrapes the 1000 pegs from http://www.rememberg.com/Peg-list-1000/


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import random
import re
import sys
import time

import requests

from sorno import loggingutil


_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


class App(object):
    MAIN_PAGE_URL = "http://www.rememberg.com/Peg-list-1000/"

    def run(self):
        # a list of 2-tuple. Each 2-tuple is (number, word)
        pegs = []

        # scrape the pegs from the initial page
        _LOG.info("Fetch from %s", self.MAIN_PAGE_URL)
        page = requests.get(self.MAIN_PAGE_URL).text
        pegs.extend(
            self.scrape_pegs(
                page,
                1,
                50,
            )
        )

        while True:
            res = self.get_next_page_url(page)
            if res is None:
                break

            url, start, end = res
            _LOG.info("Fetch from %s", url)
            page = requests.get(url).text
            pegs.extend(self.scrape_pegs(page, start, end))

        for index, word in pegs:
            print("%-3s. %s" % (index, word))

    def scrape_pegs(self, page_html, starting_peg_number, ending_peg_number):
      _LOG.info(
          "Scraping for pegs from %s to %s",
          starting_peg_number,
          ending_peg_number,
      )

      pegs = []

      for line in page_html.split("\n"):
          line = line.strip()
          matched = re.match(r"<big>(\d+)\.</big>.*<strong>(.*)</strong>", line)
          if matched:
              pegs.append(matched.groups())

      return pegs

    def get_next_page_url(self, page_html):
      """
      Get the url that links to the subsequent page, given the current page html
      in str

      Args:
          str, page_html
      Return:
          If the next page url is found, returns (str, int, int), a 3-tuple of
          url, starting number, and ending number. Otherwise, None.

      """
      for line in page_html.split("\n"):
          line = line.strip()
          # shortcut for matching before checking the regex
          if line.startswith('<a href="/Peg-list-1000/'):
              matched = re.match('<a href="(.*?)"><strong>Next:', line)
              if matched:
                  next_url = matched.groups()[0]
                  _LOG.debug("Next url is found: %s", next_url)
                  next_url = "http://www.rememberg.com" + next_url
                  start_and_end_matched = re.match(
                      r".*/peg-(\d+)-to-(\d+)",
                      next_url,
                  )
                  if start_and_end_matched:
                      start, end = start_and_end_matched.groups()
                      return next_url, start, end
                  else:
                      _LOG.warn("Start and end not found in line: %s", line)
              elif "Next" in line:
                  _LOG.warn("Unexpected missing url in line: %s", line)

      return None


def parse_args(cmd_args):
    description = """
A script to scrape the 1000 pegs from http://www.rememberg.com/Peg-list-1000/
    """
    parser = argparse.ArgumentParser(
        description=description,
        # formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = loggingutil.create_plain_logger("PLAIN")

    app = App()
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
sorno_compress_photos.py

Compress a bunch of photos in a folder. The compressed photos will be saved in
a folder called "compressed" in the same folder as the source folder.
The destination folder structure is the same as the source.

This script needs the PIL module some jpeg decoder. For mac, you may try the
following:

brew install libjpeg
sudo pip install pillow

   Copyright 2014 Heung Ming Tai

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

import logging

import os

import sys
import subprocess

from PIL import Image

_LOG = logging.getLogger(__name__)
_PLAIN_LOGGER = None  # will be created in main()


class App(object):
    def __init__(
        self,
        src_folder,
        src_file_extensions,
        quality,
    ):
        self.src_folder = os.path.realpath(src_folder)
        self.src_file_extensions = set(src_file_extensions)
        self.quality = quality
        self.dest_folder = os.path.join(
            os.path.dirname(src_folder),
            "compressed",
        )

    def run(self):
        if not os.path.exists(self.dest_folder):
            os.mkdir(self.dest_folder)

        for root, dirs, files in os.walk(self.src_folder):
            for d in dirs:
                dirpath = os.path.join(root, d)
                dest_dirpath = self.dest_folder + dirpath[len(self.src_folder):]
                if not os.path.exists(dest_dirpath):
                    _LOG.info("mkdir %s", dest_dirpath)
                    os.mkdir(dest_dirpath)

            for f in files:
                filename, ext = os.path.splitext(f)
                ext = ext.lstrip(".")
                if ext in self.src_file_extensions:
                    filepath = os.path.join(root, f)
                    filepath_without_ext = os.path.join(root, filename)
                    _LOG.info("Process file: %s", filepath)
                    image = Image.open(filepath)
                    new_filepath = (
                        self.dest_folder + filepath_without_ext[
                            len(self.src_folder):
                        ] + ".jpg"
                    )
                    _LOG.info("Save to new file: %s", new_filepath)
                    image.save(new_filepath, quality=self.quality)

#
# logger-related functions
#

def setup_logger(
    logger,
    debug=False,
    stdout=False,
    log_to_file=None,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
):
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
    )
    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)

    if log_to_file is not None:
        init_command = 'mkdir -p %s' % os.path.dirname(log_to_file)

        subprocess.check_call(init_command, shell=True)

        hdlr = TimedRotatingFileHandler(
            log_to_file,
            when="midnight",
            interval=1,
        )
        hdlr.setFormatter(formatter)
        logger.addHandler(hdlr)


def create_plain_logger(logger_name, stdout=True):
    plain_logger = logging.getLogger(logger_name)
    plain_logger.propagate = False
    plain_logger.setLevel(logging.INFO)

    if stdout:
        out = sys.stdout
    else:
        out = sys.stderr

    handler = logging.StreamHandler(stream=out)
    handler.setFormatter(
        logging.Formatter(
            fmt="%(message)s",
            datefmt="%Y",  # does not matter
        )
    )

    plain_logger.addHandler(handler)
    return plain_logger


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(formatter=None, stdout=False):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)
    return hdlr

#
# end of logger-related functions
#

def parse_args(cmd_args):
    description = """
Compress a bunch of photos in a folder.
    """
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "src_folder",
        help="Recursively traverse all photos in the folder to compress them",
    )
    parser.add_argument(
        "--file-extensions",
        default="jpg",
        help="""
            A comma separated list of file extensions. Only files with these
            extensions will be compressed. Default: %(default)s
        """
    )
    parser.add_argument(
        "--quality",
        default=90,
        help="""
            The quality retained in the compression. It's a number from 0 to
            100. The default is %(default)s
        """,
        type=int,
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _PLAIN_LOGGER

    args = parse_args(sys.argv[1:])

    setup_logger(_LOG, debug=args.debug)
    _PLAIN_LOGGER = create_plain_logger("PLAIN")

    app = App(
        args.src_folder,
        [fs.strip() for fs in args.file_extensions.split(",")],
        quality=args.quality,
    )
    app.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""sorno_cloud_vision.py makes using the Google Cloud Vision API easier.

Doc: https://cloud.google.com/vision/docs

The script generates requests for the given photos, sends the requests to Cloud
Vision, then puts the results into the corresponding response files.

    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import base64
import httplib2
import json
import logging
import os
import sys

from apiclient import discovery
import humanfriendly
from oauth2client import client as oauthclient
from sorno import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class CloudVisionApp(object):
    """A console application to do work"""
    def __init__(
        self,
        args,
    ):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args
        self.http = None
        # Endpoint for accessing the cloud vision api with api key
        self.endpoint = None
        # Google api client
        self.service = None

    def run(self):
        use_api_key = False

        self.http = httplib2.Http()

        if self.args.api_key:
            self.endpoint = (
                "https://vision.googleapis.com/v1/images:annotate?key=" +
                    self.args.api_key
            )
            use_api_key = True

        if not use_api_key:
            if not os.getenv("GOOGLE_APPLICATION_CREDENTIALS"):
                _plain_error_logger.fatal(
                    "You need to specify --api-key or set the"
                    " environment variable GOOGLE_APPLICATION_CREDENTIALS."
                    " See https://cloud.google.com/vision/docs/getting-started."
                )
                sys.exit(1)

            credentials = oauthclient.GoogleCredentials.get_application_default(
            ).create_scoped(
                 ['https://www.googleapis.com/auth/cloud-platform']
            )
            credentials.authorize(self.http)

            api_discovery_file = (
                'https://vision.googleapis.com/$discovery/rest?version=v1'
            )
            self.service = discovery.build(
                'vision',
                'v1',
                http=self.http,
                discoveryServiceUrl=api_discovery_file,
            )

        getsize = os.path.getsize
        size_limit = None
        if self.args.size_limit:
            size_limit = humanfriendly.parse_size(self.args.size_limit)
            _log.info("Size limit is %s bytes", "{:,}".format(size_limit))

        for f in self.args.files:
            _log.info("Process photo: %s", f)

            if size_limit:
                size = getsize(f)
                _log.info(
                    "File size is %s, greater than the limit %s, skipped",
                    "{:,}".format(size),
                    "{:,}".format(size_limit),
                )
                continue

            with open(f, "rb") as image:
                image_content = base64.b64encode(image.read())
                if use_api_key:
                    resp_content = self._process_with_api_key(image_content)
                else:
                    resp_content = self._process_with_credentials(image_content)
                output_filepath = f + ".output.txt"
                _log.info("Writing output for %s to %s", f, output_filepath)
                with open(output_filepath, "w") as output:
                    output.write(
                        resp_content.decode("unicode-escape").encode("utf8")
                    )

        return 0

    def _process_with_api_key(self, image_content):
        service_request_json = json.dumps(
            self._create_request_body(image_content)
        )
        if self.args.debug:
            self._write_request_for_debug(service_request_json)

        (resp, content) = self.http.request(
            self.endpoint,
            "POST",
            headers={
                'Content-Type': "application/json",
            },
            body=service_request_json,
        )
        _log.info("Response: %s", resp)
        return content

    def _process_with_credentials(self, image_content):
        request = self._create_request_body(image_content)
        if self.args.debug:
            self._write_request_for_debug(json.dumps(request, indent=4))

        client_request = self.service.images().annotate(body=request)

        response = client_request.execute()
        return json.dumps(response, indent=4)

    def _write_request_for_debug(self, request_json):
        request_filepath = "cloud-vision-request.json"
        _log.info("Writing request to %s", request_filepath)
        with open(request_filepath, "w") as f:
            f.write(request_json)

    def _create_request_body(self, image_content):
        return {
            'requests': [
                {
                    'image': {
                        'content': image_content
                    },
                    'features': [
                        {
                            'type': "LABEL_DETECTION",
                            "maxResults": 20,
                        },
                        {
                            'type': "TEXT_DETECTION",
                            "maxResults": 5,
                        },
                        # {
                        #     'type': "FACE_DETECTION",
                        #     "maxResults": 10,
                        # },
                        {
                            'type': "LANDMARK_DETECTION",
                            "maxResults": 3,
                        },
                        {
                            'type': "LOGO_DETECTION",
                            "maxResults": 5,
                        },
                        {
                            'type': "SAFE_SEARCH_DETECTION",
                            "maxResults": 10,
                        },
                        # {
                        #     'type': "IMAGE_PROPERTIES",
                        #     "maxResults": 10,
                        # },
                    ]
                },
            ]
        }


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    parser.add_argument(
        "--api-key",
    )

    parser.add_argument(
        "--size-limit",
        help="Skip files which have size higher than this limit."
        " Cloud Vision only supports up to 4MB. You can use any string that"
        " represents a size like '4MB', '3K', '5G'.",
    )

    parser.add_argument(
        "files",
        metavar='file',
        nargs="+",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = CloudVisionApp(
        args,
    )
    sys.exit(app.run())


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
"""A script that converts the mulitple inputs of a function to a single
Params Autovalue class.

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --output_autovalue_class

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --convert_method_calls_in_test_file sample_convert_method_calls_in_test_file.txt

Usage: python3 sorno_input_params_autovalue_converter.py sample_function_defs.txt --convert_call_site_file sample_convert_call_site_file.txt

    Copyright 2020 Heung Ming Tai
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
        http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

import re
import sys
import subprocess
from collections import namedtuple


fields = ('type', 'name', 'default_value')
Param = namedtuple('Param', fields, defaults=(None,) * len(fields))

AUTO_VALUE_CLASS_NAME = 'Params'

AUTO_VALUE_GET_METHOD_TEMPLATE = 'public abstract {type} {method_name}();'

AUTO_VALUE_SET_METHOD_TEMPLATE = 'public abstract Builder {method_name}({type} {param_name});'
AUTO_VALUE_CLASS_TEMPLATE = """\
import com.google.auto.value.AutoValue;

/** Input parameters for {func_name}. */
@AutoValue
public abstract static class Params {{
  {get_methods_defs}

  public static Builder builder() {{
      // TODO
      return new AutoValue_Params.Builder(){build_default_calls};
  }}

  abstract Builder toBuilder();

  /** Builder for {{@link Params}} */
  @AutoValue.Builder
  public abstract static class Builder {{
    {set_methods_defs}

    public abstract Params build();
  }}
}}
"""


class App(object):
    def __init__(self, args):
      self.args = args
      self.func_def_input_file = args.func_def_input_file

    def run(self):
      func_def_str = open(self.func_def_input_file).read()
      func_name = get_func_name(func_def_str)
      params = get_params(func_def_str)

      if self.args.output_autovalue_class:
        print_autovalue_class(func_name, params)

      if self.args.convert_method_calls_in_test_file:
        content = open(self.args.convert_method_calls_in_test_file).read()
        print(replace_content_for_test_file(func_name, params, content))

      if self.args.convert_call_site_file:
        content = open(self.args.convert_call_site_file).read()
        print(replace_content_for_call_site_file(func_name, params, content))

      return 0


def get_func_name(s):
  pre_open_paren_str = s.split('(')[0]
  if ' ' not in pre_open_paren_str:
    return pre_open_paren_str

  return pre_open_paren_str[pre_open_paren_str.rindex(' ') + 1:]


def get_params(s):
  post_open_paren_str = s.split('(', 1)[1]
  params_str = get_until_close_paren(post_open_paren_str)
  params_strs = [s.strip() for s in params_str.split(',')]

  params = []
  for param_str in params_strs:
    # Need to prevent the case that the type and parameter names are separated
    # by newlines or more than one spaces, so replace all whitespaces to
    # single space.
    param_fragments = re.sub(r'\s+', ' ', param_str).split(' ')
    t, param_name = param_fragments[-2], param_fragments[-1]
    if t == 'Boolean':
      t = 'boolean'

    if t == 'boolean':
      param = Param(type=t, name=param_name, default_value='false')
    else:
      param = Param(type=t, name=param_name)

    params.append(param)

  return params


def get_until_close_paren(s):
  open_paren_count = 0
  for i in range(len(s)):
    c = s[i]
    if c == ')':
      if open_paren_count:
        open_paren_count -= 1
      else:
        return s[:i]
    elif c == '(':
      open_paren_count += 1

  # no idea what to do, so just return s
  return s


def print_autovalue_class(func_name, params):
  get_methods_defs_strs = []
  set_methods_defs_strs = []
  build_default_calls_strs = []
  for param in params:
    get_methods_defs_strs.append(AUTO_VALUE_GET_METHOD_TEMPLATE.format(
        type=param.type,
        method_name=param.name,
    ))

    set_method_name = 'set' + upperfirst(param.name)
    set_methods_defs_strs.append(AUTO_VALUE_SET_METHOD_TEMPLATE.format(
        method_name=set_method_name,
        type=param.type,
        param_name=param.name,
    ))

    if param.default_value:
      build_default_calls_strs.append('.{set_method_name}({value})'.format(
          set_method_name=set_method_name,
          value=param.default_value,
      ))

  get_methods_defs = '\n'.join(['\n  ' + s for s in get_methods_defs_strs])
  set_methods_defs = '\n'.join(['\n    ' + s for s in set_methods_defs_strs])

  build_default_calls = ''.join(['\n    ' + s for s in build_default_calls_strs])

  print(AUTO_VALUE_CLASS_TEMPLATE.format(
      func_name=func_name,
      get_methods_defs=get_methods_defs,
      set_methods_defs=set_methods_defs,
      build_default_calls=build_default_calls,
  ))


def upperfirst(s):
  return s[0].upper() + s[1:]


def replace_content_for_test_file(func_name, params, content):
  func_name_anchor = '.' + func_name + '('
  output = ""
  while func_name_anchor in content:
    # E.g. "abcd blah.func(f1, f2, f3) hello"
    # is splitted to "abcd blah" and "f1, f2, f3) hello"
    pre, post = content.split(func_name_anchor, 1)

    # E.g. "f1, f2, f3"
    input_args_str = get_until_close_paren(post)
    replaced_input_args = replace_test_input_args_with_autovalue_class(input_args_str, params)
    output += pre + func_name_anchor + replaced_input_args
    content = post[len(input_args_str):]

  # need to append the remaining content which does not have the func name
  # anchor
  output += content

  return output


def replace_content_for_call_site_file(func_name, params, content):
  func_name_anchor = '.' + func_name + '('
  output = ""
  while func_name_anchor in content:
    # E.g. "abcd blah.func(f1, f2, f3) hello"
    # is splitted to "abcd blah" and "f1, f2, f3) hello"
    pre, post = content.split(func_name_anchor, 1)

    # E.g. "f1, f2, f3"
    input_args_str = get_until_close_paren(post)
    replaced_input_args = replace_input_args_with_autovalue_class(input_args_str, params)
    output += pre + func_name_anchor + replaced_input_args
    content = post[len(input_args_str):]

  # need to append the remaining content which does not have the func name
  # anchor
  output += content

  return output


def replace_test_input_args_with_autovalue_class(input_args_str, params):
  input_arg_strs = parse_input_arg_strs(input_args_str)
  output = AUTO_VALUE_CLASS_NAME + ".builder()"
  cleaned_arg_strs = [clean_input_arg(s) for s in input_arg_strs]

  for arg, param in zip(cleaned_arg_strs, params):
    if can_skip_arg(arg):
      continue
    output += "\n.set%s(%s)" % (upperfirst(param.name), arg)
  output += "\n.build()"
  return output


def replace_input_args_with_autovalue_class(input_args_str, params):
  input_arg_strs = parse_input_arg_strs(input_args_str)
  output = AUTO_VALUE_CLASS_NAME + ".builder()"
  cleaned_arg_strs = [clean_input_arg(s) for s in input_arg_strs]

  for arg, param in zip(cleaned_arg_strs, params):
    output += "\n.set%s(%s)" % (upperfirst(param.name), arg)
  output += "\n.build()"
  return output


def parse_input_arg_strs(s):
  input_arg_strs = []
  sofar = []
  num_open_parens = 0
  for c in s:
    if c == ',' and not num_open_parens:
      input_arg_strs.append(''.join(sofar))
      sofar.clear()
      continue

    sofar.append(c)
    if c == '(':
      num_open_parens += 1
    if c == ')':
      num_open_parens -= 1

  if sofar:
    input_arg_strs.append(''.join(sofar))

  return input_arg_strs


def clean_input_arg(s):
  if "*/" in s:
    s = s.split("*/", 1)[1]

  return s.strip()


def can_skip_arg(arg):
  return arg in ("false", "Optional.empty()")


def parse_args(cmd_args):
  description = __doc__.split("Copyright 2018")[0].strip()

  parser = argparse.ArgumentParser(
    description=description,
    formatter_class=argparse.RawDescriptionHelpFormatter,
  )
  parser.add_argument("func_def_input_file", help="A file with the function definition")
  parser.add_argument("--output_autovalue_class", action="store_true")
  parser.add_argument("--convert_method_calls_in_test_file")
  parser.add_argument("--convert_call_site_file")

  args = parser.parse_args(cmd_args)
  return args


def main():
  args = parse_args(sys.argv[1:])

  app = App(args)
  sys.exit(app.run())


if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""Prints the class dependency graph given a bunch of java source files.

This script depends on the library sorno-py-scripts. You can find out the
installation detail in https://github.com/hermantai/sorno-py-scripts.


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import collections
import logging
import os
import re
import sys

from sorno import loggingutil


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class Graph(object):
    def __init__(self):
        self.nodes = set()
        self.edges = collections.defaultdict(set)

    def add_node(self, n):
        self.nodes.add(n)

    def add_edge(self, src, dst):
        self.nodes.add(src)
        self.nodes.add(dst)

        self.edges[src].add(dst)

    def get_edges(self, n):
        return [(n, d) for d in self.edges[n]]


class App(object):
    """A console application to do work"""
    def __init__(self, args):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args

    def run(self):
        """The entry point of the script
        """
        m = self.create_classes_to_contents_map(self.args.path)
        g = self.extract_class_dependency_graph(m)
        indegrees = self.get_indegrees(g)
        if self.args.edges:
            for n in g.nodes:
                for _, dst in g.get_edges(n):
                    print(n, "->", dst)
        else:
            self.print_graph(g, indegrees)

        return 0

    def create_classes_to_contents_map(self, paths):
        m = {}
        for path in paths:
            if os.path.isdir(path):
                for root, dirs, files in os.walk(path):
                    for f in files:
                        self._set_class_to_content(m, os.path.join(root, f))
            else:
                self._set_class_to_content(m, path)

        return m

    def _set_class_to_content(self, classes_to_contents_map, filepath):
        name, ext = os.path.splitext(filepath)
        if ext != ".java":
            return

        classes_to_contents_map[os.path.basename(name)] = open(filepath).read()

    def extract_class_dependency_graph(self, classes_to_contents_map):
        g = Graph()
        for c in classes_to_contents_map:
            g.add_node(c)

        for c, content in classes_to_contents_map.iteritems():
            for n in g.nodes:
                if c == n:
                    continue
                if re.search(r"\b%s\b" % n, content):
                    g.add_edge(c, n)

        return g

    def get_indegrees(self, graph):
        indegrees = collections.defaultdict(lambda: 0)
        for n in graph.nodes:
            for _, dst in graph.get_edges(n):
                indegrees[dst] += 1

        return indegrees

    def print_graph(self, graph, indegrees):
        for n in graph.nodes:
            if indegrees[n]:
                continue

            print(n)
            self.print_edges(graph, n, 4)
            print("-" * 40)
        return

    def print_edges(self, graph, node, indent=0):
        for _, dst in graph.get_edges(node):
            print(" " * indent + "->", dst)
            self.print_edges(graph, dst, indent + 4)

def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--edges",
        action="store_true",
        help="Print the edges instead of the graph",
    )
    parser.add_argument(
        "path",
        nargs="+",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



"""Utilities to deal with Git
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import git

def get_config_reader(filepath):
  """Gets an instance of GitConfigParser

  https://github.com/gitpython-developers/GitPython/blob/77e47bc313e42f9636e37ec94f2e0b366b492836/git/config.py#L194
  """
  return git.Repo(filepath).config_reader()

def get_sections(filepath):
  """Gets the sections from git config."""
  reader = get_config_reader(filepath)
  sections = {}
  for sec_name in reader.sections():
    sections[sec_name] = reader.items_all(sec_name)

  return sections

def get_remote_urls(filepath):
  sections = get_sections(filepath)

  urls = []
  for sec, section in sections.items():
    if sec.startswith("remote"):
      for option, values in section:
        if option == 'url':
          urls.extend(values)

  return urls

def get_remote_urls_safe(filepath):
  """Safe version of get_remote_urls

  Just like get_remote_urls but returns an empty list of
  urls instead ot throwing an error if the file path is not a git repo.

  Let's say "~/py-ws/sorno-py-scripts" is a legit git repo:

  print(get_remote_urls_safe(os.path.expanduser('~/py-ws/sorno-py-scripts')))
  print(get_remote_urls_safe(os.path.expanduser('~/py-ws')))
  print(get_remote_urls_safe(os.path.expanduser('~/blah')))

  Output:
  ['https://github.com/hermantai/sorno-py-scripts.git']
  []
  []
  """

  try:
    return get_remote_urls(filepath)
  except git.exc.InvalidGitRepositoryError:
    return []
  except git.exc.NoSuchPathError:
    return []
